/*
   Copyright (C) 2002 Jason Stechschulte <stech@enormousplow.com>

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License.  This program is
	distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

#include "diver.h"

#define FORWARD_LIMIT 35
#define TERMINAL_VELOCITY 150
#define DECELERATION 15
#define ACCELERATION 40
#define UNDER_CHUTE_FORWARD 35
#define UNDER_CHUTE_VERTICAL 45
#define START_SPEED 80

Diver::Diver(Images &graphics, int x, int y, int random) {
	string filename;
	//Set the position on screen
	xpos = x;
	ypos = y;
	startx = x;
	starty = y;

	hspeed = START_SPEED;
	vspeed = START_SPEED;
	jumped = 0;
	jumpPos = random;

	next = NULL;

	// Keep a pointer to the array of images
	images = &graphics;

	// Make it invisible
	visible = 0;

	// Find the first frame for fred
	int found = 0;
	for(int thisFrame = 0; thisFrame < images->getNumberOf(); thisFrame++) {
		if(images->sprite[thisFrame].name == "fred" && !found) {
			firstFrame = thisFrame;
			found = 1;
		}
	}
	image = firstFrame;
}

Diver::~Diver() {
}
	
void Diver::draw(Window &window) {
	if(visible) {
		window.drawIMG(images->sprite[image].image, int(xpos), int(ypos), 
							 images->sprite[image].image->w, images->sprite[image].image->h);
	}
}

int Diver::move(Window &window, Plane &plane, float lag) {
	if(visible) {
		if(hspeed > FORWARD_LIMIT && !image) {
			// After jumping forward speed should decrease to forward minimum
			hspeed -= DECELERATION * lag;
		}
		if(vspeed < TERMINAL_VELOCITY && !image) {
			// After jumping vertical speed should increase to terminal velocity
			vspeed += ACCELERATION * lag;
		}
		else if(image) {
			// When chute is open, movement should be slower
			hspeed = UNDER_CHUTE_FORWARD;
			vspeed = UNDER_CHUTE_VERTICAL;
		}

		xpos += hspeed * lag;
		ypos += vspeed * lag;

		// If Fred is still on the screen, return 1, else return 0
		return ypos <= window.getHeight() ? 1 : 0;
	}
	else {
		return 1;
	}
}

int Diver::openChute(Uint16 &mouseX, Uint16 &mouseY) {
	// See if the player clicked on Fred
	if(image == 0 && visible && mouseX > xpos && mouseX < images->sprite[image].image->w + xpos &&
						  mouseY > ypos && mouseY < images->sprite[image].image->h + ypos) {
		// Then open the chute
		image = 1;
		return 1;
	}
	return 0;
}

int Diver::chuteIsOpen() {
	return image;
}

void Diver::reset(int random) {
	xpos = startx;
	ypos = starty;
	image = firstFrame;
	jumped = 0;
	visible = 0;
	hspeed = START_SPEED;
	vspeed = START_SPEED;
	jumpPos = random;
}

void Diver::makeInvisible() {
	visible = 0;
}

Diver *Diver::getNext() {
	return next;
}

void Diver::setNext(Diver *nextFred) {
	next = nextFred;
}

void Diver::attemptJump(int planePos) {
	if(!jumped && planePos >= jumpPos) {
		visible = 1;
		jumped = 1;
		xpos = planePos;
	}
}

