/*
	Copyright (C) 2002 Jason Stechschulte <stech@enormousplow.com>

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License.  This program is
	distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

#include "images.h"

Images::Images(string DATADIR) {
	string filename;
	string name;
	int numImages = 0;


	filename = DATADIR + "image.dat";

	// First count the number of images
	ifstream in(filename.c_str());
	if(!in.is_open()) {
		cerr << "Could not open " << filename << endl;
		throw;
	}
	getline(in, name);
	while(!in.eof()) {
		numImages++;
		getline(in, name);
	}
	in.close();

	// Now we load them
	sprite = new ImageData[numImages];
	numberof = numImages;
	numImages = 0;
	ifstream fin(filename.c_str());
	if(!fin.is_open()) {
		cerr << "Could not open " << filename << endl;
		throw;
	}
	while(fin >> filename) {
		// Get name
		fin >> sprite[numImages].name;
		filename = DATADIR + filename;
		if(DEBUG) cout << "Attempting to load " << filename << endl;
		// Load image
		if((sprite[numImages].image = IMG_Load(filename.c_str())) == NULL) {
			cerr << SDL_GetError() << endl;
			throw;
		}
		if((sprite[numImages].image = SDL_DisplayFormat(sprite[numImages].image)) == NULL) {
			cerr << SDL_GetError() << endl;
			throw;
		}
		// Set background color
		if(SDL_SetColorKey(sprite[numImages].image, SDL_SRCCOLORKEY, 
							 SDL_MapRGB(sprite[numImages].image->format, 0, 0, 255)) < 0) {
			cerr << "Could not set color key: " << SDL_GetError() << endl;
			throw;
		}
		numImages++;
	}
	fin.close();
}

Images::~Images() {
	for(int i = 0; i < numberof; i++) {
		SDL_FreeSurface(sprite[i].image);
	}
}

int Images::find(string image) {
	for(int thisFrame = 0; thisFrame < numberof; thisFrame++) {
		if(sprite[thisFrame].name == image) {
			return thisFrame;
		}
	}
	// If we get here, image was not found
	cerr << "Could not find image for " << image << endl;
	throw;
	return 0;
}

int Images::getNumberOf() {
	return numberof;
}
