/*
   Copyright (C) 2002 Jason Stechschulte <stech@enormousplow.com>

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License.  This program is
	distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

#include "window.h"

#define SCREEN_WIDTH 558
#define SCREEN_HEIGHT 385
#define BPP 16

Window::Window() {
	paused = 0;

	// Initialize the video
	if(DEBUG) cout << "Initializig video" << endl;
	if(SDL_Init(SDL_INIT_VIDEO) < 0) {
		cerr << "Error: could not initialize  SDL: " << SDL_GetError() << endl;
		throw;
	}
	atexit(SDL_Quit);

	// Initialize TTF
	if(DEBUG) cout << "Initializing SDL_ttf" << endl;
	if(TTF_Init() < 0) {
		cerr << "Couldn't initialize SDL_ttf: " << SDL_GetError() << endl;
		throw;
	}
	atexit(TTF_Quit);

	// Set video mode
	if(DEBUG) cout << "Setting video mode" << endl;
	if((screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, BPP, SDL_SWSURFACE)) == NULL) {
		cerr << "Couldn't set video mode: " << SDL_GetError() << endl;
		throw;
	}

	// Give the window a title
	SDL_WM_SetCaption("Free Fallin' Fred: Open His Chute Or He'll Be Dead!", "");
}

Window::~Window(void) {
}

// Blit an image on the screen surface
void Window::drawIMG(SDL_Surface *img, int x, int y) {
	SDL_Rect dest;
	dest.x = x;
	dest.y = y;
	if(SDL_BlitSurface(img, NULL, screen, &dest) < 0) {
		cerr << "Could not draw the image " << SDL_GetError() << endl;
		throw;
	}
}

// Faster blitting of an image on the screen surface
void Window::drawIMG(SDL_Surface *img, int x, int y, int w, int h) {
	// The rectangle of the image
	SDL_Rect iArea;
	// The area to start drawing
	SDL_Rect iStart;
	iArea.x = 0;
	iArea.y = 0;
	iArea.w = w;
	iArea.h = h;
	
	iStart.x = x;
	iStart.y = y;
	if(SDL_BlitSurface(img, &iArea, screen, &iStart) < 0) {
		cerr << "Could not draw the image " << SDL_GetError() << endl;
		throw;
	}
}

// Draw the background given
void Window::drawBG(SDL_Surface *image) {
	drawIMG(image, 0, 0);
}

void Window::drawBG() {
	if(SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 44, 152, 247)) < 0) {
		cerr << "Could not draw the background " << SDL_GetError() << endl;
		throw;
	}
}

void Window::drawScene() {
	if(SDL_Flip(screen) < 0) {
		cerr << "Could not draw the scene " << SDL_GetError() << endl;
		throw;
	}
}

int Window::getHeight() {
	return SCREEN_HEIGHT;
}

int Window::getWidth() {
	return SCREEN_WIDTH;
}

void Window::pause(Images &images) {
	// Find paused image
	if(paused == 0) {
		paused = images.find("pause");
	}

	SDL_Surface *black;
	Uint32 flags = SDL_SRCALPHA | SDL_SWSURFACE;
	if((black = SDL_CreateRGBSurface(flags, screen->w, screen->h, screen->format->BitsPerPixel,
									        screen->format->Rmask, screen->format->Gmask,
											  screen->format->Bmask, screen->format->Amask)) == NULL) {
		cerr << "Could not create black surface" << SDL_GetError() << endl;
		throw;
	}
	if(SDL_FillRect(black, NULL, SDL_MapRGB(screen->format, 0, 0, 0)) < 0) {
		cerr << "SDL_FillRect error: " << SDL_GetError() << endl;
		throw;
	}
	if(SDL_SetAlpha(black, SDL_SRCALPHA, 150) < 0) {
		cerr << "Could not set alpha " << SDL_GetError() << endl;
		throw;
	}
	if(SDL_BlitSurface(black, NULL, screen, NULL) < 0) {
		cerr << "Could not darken screen " << SDL_GetError() << endl;
		throw;
	}
	SDL_FreeSurface(black);
	SDL_Rect dest;
	// This centers it on the screen
	dest.x = (SCREEN_WIDTH - images.sprite[paused].image->w) / 2;
	dest.y = (SCREEN_HEIGHT - images.sprite[paused].image->h) / 2;
	if(SDL_BlitSurface(images.sprite[paused].image, NULL, screen, &dest) < 0) {
		cerr << "could not draw paused on screen " << SDL_GetError() << endl;
	}
	SDL_Flip(screen);
	// Now wait for them to unpause game
	int done = 0;
	SDL_Event event;
   while(!done && SDL_WaitEvent(&event)) {
		if(event.type == SDL_QUIT) {
			done = 1;
		}
		else if(event.type == SDL_KEYDOWN) {
			if(event.key.keysym.sym == SDLK_p) {
				done = 1;
			}
		}
	}
}
